@@ -92,6 +92,8 @@ class Address(dict):
 
 
 class AddressList(dict):
+    AddressCls = Address
+
     def __init__(self, path, rpc):
         super().__init__()
         self.path = path
@@ -99,7 +101,7 @@ class AddressList(dict):
         file_exists = False
         if os.path.isfile(self.path):
             try:
-                addresses = read_csv(self.path, Address, self.rpc)
+                addresses = read_csv(self.path, self.AddressCls, self.rpc)
                 # dict allows faster lookups
                 for addr in addresses:
                     self[addr.address] = addr
@@ -110,11 +112,25 @@ class AddressList(dict):
 
     def save(self):
         if len(list(self.keys())) > 0:
-            write_csv(self.path, list(self.values()), Address)
+            write_csv(self.path, list(self.values()), self.AddressCls)
         self._file_exists = True
 
+    def set_labels(self, arr):
+        """
+        Sets the labels for existing addresses
+
+        arr should be a list with dicts, example:
+            [{'address':'bc1qabc...', 'label':'mylabel1'}, {'address':'bc1qabd...', 'label':'mylabel2'}, ...]
+        """
+        for addr in arr:
+            if addr["address"] in self:
+                self[addr["address"]].set_label(addr["label"])
+
     def add(self, arr, check_rpc=False):
-        """arr should be a list of dicts"""
+        """
+        arr should be a list with dicts, example:
+            [{'address':'bc1qabc...', 'label':'mylabel1'}, {'address':'bc1qabd...', 'label':'mylabel2'}, ...]
+        """
         labeled_addresses = {}
         if check_rpc:
             # get all available labels
@@ -135,11 +151,11 @@ class AddressList(dict):
             # if we found a label for it - import
             if addr["address"] in labeled_addresses:
                 addr["label"] = labeled_addresses[addr["address"]]
-            self[addr["address"]] = Address(self.rpc, **addr)
+            self[addr["address"]] = self.AddressCls(self.rpc, **addr)
         # add all labeled addresses but not from the array (destination)
         for addr in labeled_addresses:
             if addr not in self:
-                self[addr] = Address(
+                self[addr] = self.AddressCls(
                     self.rpc,
                     address=addr,
                     label=labeled_addresses[addr],
@@ -150,7 +166,7 @@ class AddressList(dict):
 
     def set_label(self, address, label):
         if address not in self:
-            self[address] = Address(self.rpc, address=address, label=label)
+            self[address] = self.AddressCls(self.rpc, address=address, label=label)
         self[address].set_label(label)
         self.save()
 
@@ -179,8 +195,8 @@ class AddressList(dict):
 
     def max_index(self, change=False):
         return max(
-            -1,
-            -1,
+            0,
+            0,
             *[addr.index for addr in self.values() if addr.change == change],
         )
 
