@@ -38,6 +38,7 @@ class BaseConfig(object):
     SPECTER_DATA_FOLDER = os.path.expanduser(
         os.getenv("SPECTER_DATA_FOLDER", "~/.specter")
     )
+    SPECTER_API_ACTIVE = _get_bool_env_var("SPECTER_API_ACTIVE", "False")
     # Logging
     # SPECTER_LOGFILE will get created dynamically in server.py
     # using:
@@ -104,6 +105,28 @@ class BaseConfig(object):
         },
     }
 
+    # Babel integration. English listed first; other alphabetical by language code
+    LANGUAGES = {
+        "en": "English",
+        "bg": "Български",
+        "de": "Deutsch",
+        "el": "Ελληνικά",
+        "es": "Español",
+        "fr": "Français",
+        "he": "עברית",
+        "hi": "हिंदी",
+        "nl": "Nederlands",
+        "pl": "Polski",
+        "pt": "Português",
+        "ru": "Русский",
+        "sv": "Svenska",
+        "zh_Hans_CN": "简体中文",
+        "zh_Hant_TW": "繁體中文",
+    }
+
+    # Babel integration. List of languages written from right to left for RTL support in the UI
+    RTL_LANGUAGES = ["he"]
+
 
 class DevelopmentConfig(BaseConfig):
     # https://stackoverflow.com/questions/22463939/demystify-flask-app-secret-key
@@ -111,15 +134,24 @@ class DevelopmentConfig(BaseConfig):
     SPECTER_DATA_FOLDER = os.path.expanduser(
         os.getenv("SPECTER_DATA_FOLDER", "~/.specter_dev")
     )
+    # API active by default in dev-mode
+    SPECTER_API_ACTIVE = _get_bool_env_var("SPECTER_API_ACTIVE", "True")
 
 
 class TestConfig(BaseConfig):
     SECRET_KEY = "test key"
+    # This should never be used as the data-folder is injected at runtime
+    # But let's be sure before something horrible happens:
+    SPECTER_DATA_FOLDER = os.path.expanduser(
+        os.getenv("SPECTER_DATA_FOLDER", "~/.specter_testing")
+    )
+    # API active by default in test-mode
+    SPECTER_API_ACTIVE = _get_bool_env_var("SPECTER_API_ACTIVE", "True")
 
 
 class CypressTestConfig(TestConfig):
     SPECTER_DATA_FOLDER = os.path.expanduser(
-        os.getenv("SPECTER_DATA_FOLDER", "~/.specter-cypress")
+        os.getenv("SPECTER_DATA_FOLDER", "~/.specter_cypress")
     )
     PORT = os.getenv("PORT", 25444)
 
