@@ -40,6 +40,7 @@ def get_rpcconfig(datadir=get_default_datadir()):
         "cookies": [],
     }
     if not os.path.isdir(datadir):  # we don't know where to search for files
+        logger.warning(f"{datadir} not found")
         return config
     # load content from bitcoin.conf
     bitcoin_conf_file = os.path.join(datadir, "bitcoin.conf")
@@ -112,8 +113,6 @@ def get_configs(config=None, datadir=get_default_datadir()):
 
 
 def detect_rpc_confs(config=None, datadir=get_default_datadir()):
-    if config is None:
-        config = get_rpcconfig(datadir=datadir)
     rpcconfs = get_configs(config)
     rpc_arr = []
     for conf in rpcconfs:
@@ -185,7 +184,7 @@ def autodetect_rpc_confs(
 
 
 class RpcError(Exception):
-    """Specifically created for error-handling of the BitcoiCore-API
+    """Specifically created for error-handling of the BitcoinCore-API
     if thrown, check for errors like this:
     try:
         rpc.does_not_exist()
@@ -215,6 +214,9 @@ class RpcError(Exception):
 class BitcoinRPC:
     counter = 0
 
+    last_call_hash = None
+    last_call_hash_counter = 0
+
     def __init__(
         self,
         user="bitcoin",
@@ -240,6 +242,8 @@ class BitcoinRPC:
         self.proxy_url = proxy_url
         self.only_tor = only_tor
         self.r = None
+        self.last_call_hash = None
+        self.last_call_hash_counter = 0
         # session reuse speeds up requests
         if session is None:
             self._create_session()
@@ -333,17 +337,45 @@ class BitcoinRPC:
         url = self.url
         if "wallet" in kwargs:
             url = url + "/wallet/{}".format(kwargs["wallet"])
+        self.trace_call(url, payload)
         r = self.session.post(
             url, data=json.dumps(payload), headers=headers, timeout=timeout
         )
         self.r = r
         if r.status_code != 200:
+            logger.debug(f"last call FAILED: {r.text} (raising RpcError)")
             raise RpcError(
                 "Server responded with error code %d: %s" % (r.status_code, r.text), r
             )
         r = r.json()
         return r
 
+    @classmethod
+    def trace_call(cls, url, payload):
+        """logs out the call and its payload, reduces noise by suppressing repeated calls"""
+        if False:  # noise-reduction
+            logger.debug(f"call({url}) payload:{payload}")
+        else:
+            current_hash = hash(
+                json.dumps({"url": url, "payload": payload}, sort_keys=True)
+            )
+            if cls.last_call_hash == None:
+                cls.last_call_hash = current_hash
+                cls.last_call_hash_counter = 0
+            elif cls.last_call_hash == current_hash:
+                cls.last_call_hash_counter = cls.last_call_hash_counter + 1
+                return
+            else:
+                if cls.last_call_hash_counter > 0:
+                    logger.debug(f"call repeated {cls.last_call_hash_counter} times")
+                    cls.last_call_hash_counter = 0
+                    cls.last_call_hash = current_hash
+                else:
+                    cls.last_call_hash = current_hash
+            logger.debug(
+                "call({: <28}) payload:{}".format("/".join(url.split("/")[3:]), payload)
+            )
+
     def __getattr__(self, method):
         def fn(*args, **kwargs):
             r = self.multi([(method, *args)], **kwargs)[0]
