@@ -5,7 +5,8 @@ import secrets
 from pathlib import Path
 
 from dotenv import load_dotenv
-from flask import Flask, redirect, url_for
+from flask import Flask, redirect, request, url_for, jsonify, session
+from flask_babel import Babel
 from flask_login import LoginManager, login_user
 from flask_wtf.csrf import CSRFProtect
 
@@ -26,6 +27,27 @@ load_dotenv(env_path)
 csrf = CSRFProtect()
 
 
+class SpecterFlask(Flask):
+    @property
+    def supported_languages(self):
+        return self.config["LANGUAGES"]
+
+    def get_language_code(self):
+        """
+        Helper for Babel and other related language selection tasks.
+        """
+        if "language_code" in session:
+            # Explicit selection
+            return session["language_code"]
+        else:
+            # autodetect
+            return request.accept_languages.best_match(self.supported_languages.keys())
+
+    def set_language_code(self, language_code):
+        session["language_code"] = language_code
+        session["is_language_rtl"] = language_code in self.config["RTL_LANGUAGES"]
+
+
 def calc_module_name(config):
     """tiny helper to make passing configs more convenient"""
     if "." in config:
@@ -58,11 +80,13 @@ def create_app(config=None):
         template_folder = os.path.join(sys._MEIPASS, "templates")
         static_folder = os.path.join(sys._MEIPASS, "static")
         logger.info("pyinstaller based instance running in {}".format(sys._MEIPASS))
-        app = Flask(
+        app = SpecterFlask(
             __name__, template_folder=template_folder, static_folder=static_folder
         )
     else:
-        app = Flask(__name__, template_folder="templates", static_folder="static")
+        app = SpecterFlask(
+            __name__, template_folder="templates", static_folder="static"
+        )
     app.jinja_env.autoescape = select_autoescape(default_for_string=True, default=True)
     logger.info(f"Configuration: {config}")
     app.config.from_object(config)
@@ -71,6 +95,7 @@ def create_app(config=None):
     )
     csrf.init_app(app)
     app.csrf = csrf
+
     return app
 
 
@@ -136,12 +161,44 @@ def init_app(app, hwibridge=False, specter=None):
         def index():
             return redirect(url_for("hwi_server.hwi_bridge_settings"))
 
+    if app.config["SPECTER_API_ACTIVE"]:
+        app.logger.info("Initializing REST ...")
+        from cryptoadvance.specter.api import api_bp
+
+        app.register_blueprint(api_bp)
+
     @app.context_processor
     def inject_tor():
         if app.config["DEBUG"]:
             return dict(tor_service_id="", tor_enabled=False)
         return dict(tor_service_id=app.tor_service_id, tor_enabled=app.tor_enabled)
 
+    # --------------------- Babel integration ---------------------
+    if getattr(sys, "frozen", False):
+        app.config["BABEL_TRANSLATION_DIRECTORIES"] = os.path.join(
+            sys._MEIPASS, "translations"
+        )
+    babel = Babel(app)
+
+    @babel.localeselector
+    def get_language_code():
+        # Enables Babel to auto-detect current language
+        return app.get_language_code()
+
+    @app.route("/set_language", methods=["POST"])
+    def set_language_code():
+        json_data = request.get_json()
+        if (
+            "language_code" in json_data
+            and json_data["language_code"] in app.supported_languages
+        ):
+            app.set_language_code(json_data["language_code"])
+            return jsonify(success=True)
+        else:
+            return jsonify(success=False)
+
+    # --------------------- Babel integration ---------------------
+
     return app
 
 
