@@ -1,5 +1,6 @@
 import copy, hashlib, json, logging, os, re
 import time
+from collections import OrderedDict
 from .device import Device
 from .key import Key
 from .util.merkleblock import is_valid_merkle_proof
@@ -14,7 +15,7 @@ from embit.transaction import Transaction
 
 from .util.xpub import get_xpub_fingerprint
 from .util.tx import decoderawtransaction
-from .persistence import write_json_file, delete_file
+from .persistence import write_json_file, delete_file, delete_folder
 from io import BytesIO
 from .specter_error import SpecterError
 import threading
@@ -26,6 +27,27 @@ from .txlist import TxList
 logger = logging.getLogger(__name__)
 LISTTRANSACTIONS_BATCH_SIZE = 1000
 
+purposes = OrderedDict(
+    {
+        None: "General",
+        "wpkh": "Single (Segwit)",
+        "sh-wpkh": "Single (Nested)",
+        "pkh": "Single (Legacy)",
+        "wsh": "Multisig (Segwit)",
+        "sh-wsh": "Multisig (Nested)",
+        "sh": "Multisig (Legacy)",
+    }
+)
+
+addrtypes = {
+    "pkh": "legacy",
+    "sh-wpkh": "p2sh-segwit",
+    "wpkh": "bech32",
+    "sh": "legacy",
+    "sh-wsh": "p2sh-segwit",
+    "wsh": "bech32",
+}
+
 
 class Wallet:
     # if the wallet is old we import 300 addresses
@@ -35,6 +57,9 @@ class Wallet:
     # minimal fee rate is slightly above 1 sat/vbyte
     # to avoid rounding errors
     MIN_FEE_RATE = 1.01
+    # for inheritance (to simplify LWallet logic)
+    AddressListCls = AddressList
+    TxListCls = TxList
 
     def __init__(
         self,
@@ -95,12 +120,12 @@ class Wallet:
         self.last_block = last_block
 
         addr_path = self.fullpath.replace(".json", "_addr.csv")
-        self._addresses = AddressList(addr_path, self.rpc)
+        self._addresses = self.AddressListCls(addr_path, self.rpc)
         if not self._addresses.file_exists:
             self.fetch_labels()
 
         txs_path = self.fullpath.replace(".json", "_txs.csv")
-        self._transactions = TxList(
+        self._transactions = self.TxListCls(
             txs_path, self.rpc, self._addresses, self.manager.chain
         )
 
@@ -113,6 +138,124 @@ class Wallet:
         if old_format_detected or self.last_block != last_block:
             self.save_to_file()
 
+    @classmethod
+    def create(
+        cls,
+        rpc,
+        rpc_path,
+        working_folder,
+        device_manager,
+        wallet_manager,
+        name,
+        alias,
+        sigs_required,
+        key_type,
+        keys,
+        devices,
+        core_version=None,
+    ):
+        """Creates a wallet. If core_version is not specified - get it from rpc"""
+        # get xpubs in a form [fgp/der]xpub from all keys
+        xpubs = [key.metadata["combined"] for key in keys]
+        recv_keys = ["%s/0/*" % xpub for xpub in xpubs]
+        change_keys = ["%s/1/*" % xpub for xpub in xpubs]
+        is_multisig = len(keys) > 1
+        # we start by constructing an argument for descriptor wrappers
+        if is_multisig:
+            recv_descriptor = "sortedmulti({},{})".format(
+                sigs_required, ",".join(recv_keys)
+            )
+            change_descriptor = "sortedmulti({},{})".format(
+                sigs_required, ",".join(change_keys)
+            )
+        else:
+            recv_descriptor = recv_keys[0]
+            change_descriptor = change_keys[0]
+        # now we iterate over script-type in reverse order
+        # to get sh(wpkh(xpub)) from sh-wpkh and xpub
+        arr = key_type.split("-")
+        for el in arr[::-1]:
+            recv_descriptor = "%s(%s)" % (el, recv_descriptor)
+            change_descriptor = "%s(%s)" % (el, change_descriptor)
+
+        recv_descriptor = AddChecksum(recv_descriptor)
+        change_descriptor = AddChecksum(change_descriptor)
+        if not recv_descriptor != change_descriptor:
+            raise SpecterError(
+                f"The recv_descriptor ({recv_descriptor}) is the same than the change_descriptor ({change_descriptor})"
+            )
+
+        # get Core version if we don't know it
+        if core_version is None:
+            core_version = rpc.getnetworkinfo().get("version", 0)
+
+        use_descriptors = core_version >= 210000
+        if use_descriptors:
+            # Use descriptor wallet
+            rpc.createwallet(os.path.join(rpc_path, alias), True, True, "", False, True)
+        else:
+            rpc.createwallet(os.path.join(rpc_path, alias), True)
+
+        wallet_rpc = rpc.wallet(os.path.join(rpc_path, alias))
+        # import descriptors
+        args = [
+            {
+                "desc": desc,
+                "internal": change,
+                "timestamp": "now",
+                "watchonly": True,
+            }
+            for (change, desc) in [(False, recv_descriptor), (True, change_descriptor)]
+        ]
+        for arg in args:
+            if use_descriptors:
+                arg["active"] = True
+            else:
+                arg["keypool"] = True
+                arg["range"] = [0, cls.GAP_LIMIT]
+
+        if not args[0] != args[1]:
+            raise SpecterError(f"{args[0]} is equal {args[1]}")
+
+        # Descriptor wallets were introduced in v0.21.0, but upgraded nodes may
+        # still have legacy wallets. Use getwalletinfo to check the wallet type.
+        # The "keypool" for descriptor wallets is automatically refilled
+        if use_descriptors:
+            res = wallet_rpc.importdescriptors(args)
+        else:
+            res = wallet_rpc.importmulti(args, {"rescan": False})
+
+        if not all([r["success"] for r in res]):
+            all_issues = " and ".join(
+                r["error"]["message"] for r in res if r["success"] == False
+            )
+            raise SpecterError(all_issues)
+
+        return cls(
+            name,
+            alias,
+            "{} of {} {}".format(sigs_required, len(keys), purposes[key_type])
+            if len(keys) > 1
+            else purposes[key_type],
+            addrtypes[key_type],
+            "",
+            -1,
+            "",
+            -1,
+            0,
+            0,
+            recv_descriptor,
+            change_descriptor,
+            keys,
+            devices,
+            sigs_required,
+            {},
+            [],
+            os.path.join(working_folder, "%s.json" % alias),
+            device_manager,
+            wallet_manager,
+        )
+
     def fetch_labels(self):
         """Load addresses and labels to self._addresses"""
         recv = [
@@ -156,7 +299,7 @@ class Wallet:
                 [("gettransaction", txid) for txid in unconfirmed_selftransfers]
             )
         while True:
-            res = (
+            txlist = (
                 self.rpc.listtransactions(
                     "*",
                     LISTTRANSACTIONS_BATCH_SIZE,
@@ -165,9 +308,13 @@ class Wallet:
                 )
                 + [tx["result"] for tx in unconfirmed_selftransfers_txs]
             )
+            # list of transactions that we don't know about,
+            # or that it has a different blockhash (reorg / confirmed)
+            # or doesn't have an address(?)
+            # or has wallet conflicts
             res = [
                 tx
-                for tx in res
+                for tx in txlist
                 if tx["txid"] not in self._transactions
                 or not self._transactions[tx["txid"]].get("address", None)
                 or self._transactions[tx["txid"]].get("blockhash", None)
@@ -206,7 +353,7 @@ class Wallet:
         ##################### Remove from here after dropping Core v0.19 support #####################
         check_blockheight = False
         for tx in txs.values():
-            if tx.get("confirmations", 0) > 0 and "blockheight" not in tx:
+            if tx and tx.get("confirmations", 0) > 0 and "blockheight" not in tx:
                 check_blockheight = True
                 break
         if check_blockheight:
@@ -215,19 +362,55 @@ class Wallet:
                 if tx.get("confirmations", 0) > 0:
                     tx["blockheight"] = current_blockheight - tx["confirmations"] + 1
         ##################### Remove until here after dropping Core v0.19 support #####################
-        self._transactions.add(txs)
         if self.use_descriptors:
-            while (
-                len(
-                    [
-                        tx
-                        for tx in self._transactions
-                        if self._transactions[tx]["category"] != "send"
-                        and not self._transactions[tx]["address"]
+            # Get all used addresses that belong to the wallet
+            addresses_info_multi = self.rpc.multi(
+                [
+                    ("getaddressinfo", address)
+                    for address in [
+                        tx["details"][0].get("address")
+                        for tx in txs.values()
+                        if tx.get("details")
+                        and (
+                            tx.get("details")[0].get("category") != "send"
+                            and tx["details"][0].get("address") not in self._addresses
+                        )
                     ]
-                )
-                != 0
+                    if address
+                ]
+            )
+            addresses_info = [
+                r["result"]
+                for r in addresses_info_multi
+                if r["result"].get("ismine", False)
+            ]
+
+            # Gets max index used receiving and change addresses
+            max_used_receiving = (
+                self._addresses.max_index(change=False) - self.GAP_LIMIT
+            )
+            max_used_change = self._addresses.max_index(change=True) - self.GAP_LIMIT
+
+            for address in addresses_info:
+                desc = LDescriptor.from_string(address["desc"])
+                indexes = [
+                    {
+                        "idx": k.origin.derivation[-1],
+                        "change": k.origin.derivation[-2],
+                    }
+                    for k in desc.keys
+                ]
+                for idx in indexes:
+                    if int(idx["change"]) == 0:
+                        max_used_receiving = max(max_used_receiving, int(idx["idx"]))
+                    elif int(idx["change"]) == 1:
+                        max_used_change = max(max_used_change, int(idx["idx"]))
+
+            # If max receiving address bigger than current max receiving index minus the gap limit - self._addresses.max_index(change=False)
+            if max_used_receiving + self.GAP_LIMIT > self._addresses.max_index(
+                change=False
             ):
+                # Add receiving addresses until the new max address plus the GAP_LIMIT
                 addresses = [
                     dict(
                         address=self.get_address(
@@ -238,9 +421,16 @@ class Wallet:
                     )
                     for idx in range(
                         self._addresses.max_index(change=False),
-                        self._addresses.max_index(change=False) + self.GAP_LIMIT,
+                        max_used_receiving + self.GAP_LIMIT,
                     )
                 ]
+                self._addresses.add(addresses, check_rpc=False)
+
+            # If max change address bigger than current max change index minus the gap limit  - self._addresses.max_index(change=True)
+            if max_used_change + self.GAP_LIMIT > self._addresses.max_index(
+                change=True
+            ):
+                # Add change addresses until the new max address plus the GAP_LIMIT
                 change_addresses = [
                     dict(
                         address=self.get_address(idx, change=True, check_keypool=False),
@@ -249,12 +439,43 @@ class Wallet:
                     )
                     for idx in range(
                         self._addresses.max_index(change=True),
-                        self._addresses.max_index(change=True) + self.GAP_LIMIT,
+                        max_used_change + self.GAP_LIMIT,
                     )
                 ]
-                self._addresses.add(addresses, check_rpc=False)
                 self._addresses.add(change_addresses, check_rpc=False)
-                self._transactions.add(txs)
+
+        self._transactions.add(txs)
+
+    def import_electrum_label_export(self, electrum_label_export):
+        if not electrum_label_export:
+            logger.warning(f"No electrum export json provided.")
+            return
+
+        labeled_addresses = json.loads(electrum_label_export)
+
+        # write tx_label to address_label in labels
+        for txitem in self._transactions.values():
+            if txitem["txid"] not in labeled_addresses:
+                continue
+
+            address_list = (
+                [txitem["address"]]
+                if isinstance(txitem["address"], str)
+                else txitem["address"]
+            )
+            for one_address in address_list:
+                if one_address in labeled_addresses:
+                    continue  # if there is an address label, it supercedes the tx label
+                labeled_addresses[one_address] = labeled_addresses[txitem["txid"]]
+
+        # convert labeled_addresses to arr (for AddressList.set_labels) and allow only already existing addresses
+        arr = [
+            {"address": address, "label": label}
+            for address, label in labeled_addresses.items()
+            if address in self._addresses
+        ]
+        self._addresses.set_labels(arr)
+        logger.info(f"Imported {len(arr)} address labels.")
 
     def update(self):
         self.getdata()
@@ -450,7 +671,7 @@ class Wallet:
             # list only the ones we know (have descriptor for it)
             utxo = [tx for tx in utxo if tx.get("desc", "")]
             for tx in utxo:
-                tx_data = self.gettransaction(tx["txid"], 0)
+                tx_data = self.gettransaction(tx["txid"], 0, full=False)
                 tx["time"] = tx_data["time"]
                 tx["category"] = "send"
                 if "locked" not in tx:
@@ -479,7 +700,7 @@ class Wallet:
             # Could happen if address not in wallet (wallet was imported)
             # try adding keypool
             logger.info(
-                f"Didn't get transactions on address {self.change_address}. Refilling keypool."
+                f"Didn't get transactions on change address {self.change_address}. Refilling keypool."
             )
             self.keypoolrefill(0, end=self.keypool, change=False)
             self.keypoolrefill(0, end=self.change_keypool, change=True)
@@ -534,6 +755,11 @@ class Wallet:
         delete_file(self.fullpath + ".bkp")
         delete_file(self._addresses.path)
         delete_file(self._transactions.path)
+        # the folder might not exist
+        try:
+            delete_folder(self._transactions.rawdir)
+        except:
+            pass
 
     @property
     def use_descriptors(self):
@@ -555,13 +781,14 @@ class Wallet:
         for psbt in self.pending_psbts:
             amount += sum(
                 [
-                    utxo["witness_utxo"]["amount"]
+                    utxo.get("witness_utxo", {}).get("amount", 0)
+                    or utxo.get("value", 0)
                     for utxo in self.pending_psbts[psbt]["inputs"]
                 ]
             )
         return amount
 
-    def delete_pending_psbt(self, txid):
+    def delete_pending_psbt(self, txid, tx=None):
         try:
             self.rpc.lockunspent(True, self.pending_psbts[txid]["tx"]["vin"])
         except:
@@ -668,7 +895,7 @@ class Wallet:
                     not tx["conflicts"]
                     or max(
                         [
-                            self.gettransaction(conflicting_tx, 0)["time"]
+                            self.gettransaction(conflicting_tx, 0, full=False)["time"]
                             for conflicting_tx in tx["conflicts"]
                         ]
                     )
@@ -735,12 +962,15 @@ class Wallet:
             logging.error("Exception while processing txlist: {}".format(e))
             return []
 
-    def gettransaction(self, txid, blockheight=None, decode=False):
+    def gettransaction(self, txid, blockheight=None, decode=False, full=True):
+        """Gets transaction from cache
+        If full=True it will also contain "hex" key with full hex transaction.
+        If decode=True it will decode the transaction similar to Core decoderawtransaction call
+        """
         try:
-            tx_data = self._transactions.gettransaction(txid, blockheight)
-            if decode:
-                return decoderawtransaction(tx_data["hex"], self.manager.chain)
-            return tx_data
+            return self._transactions.gettransaction(
+                txid, blockheight, full=full, decode=decode
+            )
         except Exception as e:
             logger.warning("Could not get transaction {}, error: {}".format(txid, e))
 
@@ -955,15 +1185,13 @@ class Wallet:
 
     @property
     def account_map(self):
-        return (
-            '{ "label": "'
-            + self.name.replace("'", "\\'")
-            + '", "blockheight": '
-            + str(self.blockheight)
-            + ', "descriptor": "'
-            + self.recv_descriptor.replace("/", "\\/")
-            + '" }'
-        )
+        account_map_dict = {
+            "label": self.name,
+            "blockheight": self.blockheight,
+            "descriptor": self.recv_descriptor,
+            "devices": [{"type": d.device_type, "label": d.name} for d in self.devices],
+        }
+        return json.dumps(account_map_dict)
 
     def getnewaddress(self, change=False, save=True):
         if change:
@@ -988,7 +1216,7 @@ class Wallet:
                 self.keypoolrefill(pool, index + self.GAP_LIMIT, change=change)
         desc = self.change_descriptor if change else self.recv_descriptor
         return (
-            LDescriptor.from_string(desc.split("#")[0])
+            LDescriptor.from_string(desc)
             .derive(index)
             .address(get_network(self.manager.chain))
         )
@@ -1016,10 +1244,7 @@ class Wallet:
         return {"descriptor": derived_desc, "xpubs_descriptor": derived_desc_xpubs}
 
     def get_address_info(self, address):
-        try:
-            return self._addresses[address]
-        except:
-            return None
+        return self._addresses.get(address)
 
     def is_address_mine(self, address):
         addrinfo = self.get_address_info(address)
@@ -1125,12 +1350,8 @@ class Wallet:
                     available["trusted"] = round(available["trusted"], 8)
             available["untrusted_pending"] = round(available["untrusted_pending"], 8)
             balance["available"] = available
-        except:
-            balance = {
-                "trusted": 0,
-                "untrusted_pending": 0,
-                "available": {"trusted": 0, "untrusted_pending": 0},
-            }
+        except Exception as e:
+            raise SpecterError(f"was not able to get wallet_balance because {e}")
         self.balance = balance
         return self.balance
 
@@ -1170,48 +1391,14 @@ class Wallet:
         # Descriptor wallets were introduced in v0.21.0, but upgraded nodes may
         # still have legacy wallets. Use getwalletinfo to check the wallet type.
         # The "keypool" for descriptor wallets is automatically refilled
-        if (not self.use_descriptors) or start == 0:
-            if not self.is_multisig:
-                if self.use_descriptors:
-                    r = self.rpc.importdescriptors(args)
-                else:
-                    r = self.rpc.importmulti(args, {"rescan": False})
-            # bip67 requires sorted public keys for multisig addresses
-            else:
-                if self.use_descriptors:
-                    self.rpc.importdescriptors(args)
-                else:
-                    # try if sortedmulti is supported
-                    r = self.rpc.importmulti(args, {"rescan": False})
-                    # doesn't raise, but instead returns "success": False
-                    if not r[0]["success"]:
-                        # first import normal multi
-                        # remove checksum
-                        desc = desc.split("#")[0]
-                        # switch to multi
-                        desc = desc.replace("sortedmulti", "multi")
-                        # add checksum
-                        desc = AddChecksum(desc)
-                        # update descriptor
-                        args[0]["desc"] = desc
-                        r = self.rpc.importmulti(args, {"rescan": False})
-                        # make a batch of single addresses to import
-                        arg = args[0]
-                        # remove range key
-                        arg.pop("range")
-                        batch = []
-                        for i in range(start, end):
-                            sorted_desc = sort_descriptor(desc, index=i)
-                            # create fresh object
-                            obj = {}
-                            obj.update(arg)
-                            obj.update({"desc": sorted_desc})
-                            batch.append(obj)
-                        r = self.rpc.importmulti(batch, {"rescan": False})
+        if not self.use_descriptors:
+            r = self.rpc.importmulti(args, {"rescan": False})
+
         if change:
             self.change_keypool = end
         else:
             self.keypool = end
+
         self.save_to_file()
         return end
 
@@ -1289,7 +1476,7 @@ class Wallet:
             if not rbf_edit_mode:
                 if self.full_available_balance < sum(amounts):
                     raise SpecterError(
-                        "The wallet does not have sufficient funds to make the transaction."
+                        f"Wallet {self.name} does not have sufficient funds to make the transaction."
                     )
 
             if selected_coins != []:
@@ -1329,15 +1516,13 @@ class Wallet:
                 "replaceable": rbf,
             }
 
-            # 209900 is pre-v21 for Elements Core
-            if self.manager.bitcoin_core_version_raw >= 209900:
+            if self.manager.bitcoin_core_version_raw >= 210000:
                 options["add_inputs"] = selected_coins == []
 
             if fee_rate > 0:
                 # bitcoin core needs us to convert sat/B to BTC/kB
                 options["feeRate"] = round((fee_rate * 1000) / 1e8, 8)
 
-            # don't reuse change addresses - use getrawchangeaddress instead
             r = self.rpc.walletcreatefundedpsbt(
                 extra_inputs,  # inputs
                 [{addresses[i]: amounts[i]} for i in range(len(addresses))],  # output
@@ -1610,8 +1795,7 @@ class Wallet:
             if (
                 "addresses" not in out["scriptPubKey"]
                 or len(out["scriptPubKey"]["addresses"]) == 0
-                or "address" not in out["scriptPubKey"]
-            ):
+            ) and "address" not in out["scriptPubKey"]:
                 # TODO: we need to handle it somehow differently
                 raise SpecterError("Sending to raw scripts is not supported yet")
             addr = get_address_from_dict(out["scriptPubKey"])
@@ -1696,3 +1880,6 @@ class Wallet:
             )
 
         return addresses_info
+
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__} name={self.name } alias={self.alias}>"
